#include "WinFile.h"

bool WinFile::file_exists( const std::wstring& path ){

	WIN32_FIND_DATA fd = { 0 };
	HANDLE find;

	if(INVALID_HANDLE_VALUE == (find = FindFirstFile(path.c_str(),&fd))){
		FindClose(find);
		return 0;
	}
	FindClose(find);
	return 1;
}
WinFile::WinFile(void)
{
	this->isOpen = 0;
	this->Hfile = 0;
	this->attr = 0;
	this->rw = 0;
	this->LastErrorCode = 0;
}
WinFile::~WinFile(void)
{
	if(this->isOpen){
		this->isOpen = 0;
		CloseHandle(this->Hfile);
		this->Hfile = 0;
	}
}
WinFile::WinFile( WinFile& wf){
	this->Hfile = wf.Hfile;
	this->attr = wf.attr;
	this->rw = wf.rw;
	this->isOpen = wf.isOpen;
}
void WinFile::GetLastErrorText( ){
	LPVOID lpmsg = 0;
	this->LastErrorCode = ::GetLastError();
	FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
		__in_opt  NULL,
		__in     this->LastErrorCode,
		__in      MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		__out(LPWSTR)&lpmsg,
		__in      NULL,
		__in_opt  NULL
		);

	this->error = (wchar_t *)lpmsg;
	LocalFree(lpmsg);
}
bool WinFile::Fopen( const std::wstring& path, const wchar_t * method){
	if(!path.length()){
		this->error = L"Path to the file is empty";
		return 0;
	}

	if(this->isOpen){ //        
		this->isOpen = 0;
		CloseHandle(this->Hfile);
		this->Hfile = 0;
	}

	if( this->file_exists( path ) ) {
		this->attr = GetFileAttributes( path.c_str() );
	}else{
		if( method == L"r" ){
			this->error = L"File does not exist";
			return 0;
		}
		this->attr = FILE_ATTRIBUTE_NORMAL;
	}

	if(  method == L"r" ){
		this->Hfile = CreateFile(path.c_str(),GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,this->attr,NULL);
	}else{
		this->Hfile = CreateFile(path.c_str(),GENERIC_READ|GENERIC_WRITE,NULL,NULL,OPEN_ALWAYS,this->attr,NULL);
	}

	if( !this->Hfile || this->Hfile == INVALID_HANDLE_VALUE){
		this->GetLastErrorText();
		return 0;
	}

	if( method == L"w" ) {
		SetFilePointer(this->Hfile,NULL,NULL,FILE_BEGIN); 
		SetEndOfFile(this->Hfile);
	}
	if( method == L"a" ) {
		SetFilePointer(this->Hfile,NULL,NULL,FILE_END); 
	}

	this->isOpen = 1;
	return 1;
}
void WinFile::Fclose(){
	if(this->isOpen){
		this->isOpen = 0;
		CloseHandle(this->Hfile);
		this->Hfile = 0;
	}
}
bool WinFile::SetCursor(const LONGLONG &byte ,const int &start){

	if( this->isOpen ){
		LARGE_INTEGER li;
		li.QuadPart = byte;
		if(SetFilePointerEx(this->Hfile,li,NULL,start)){
			return 1;
		}
		this->LastErrorCode = ::GetLastError();
		return 0;
	}
	this->error = L"File is not open";
	return 0;

}
bool WinFile::ClearFile( void ){
	if( this->isOpen ){
		SetFilePointer(this->Hfile,NULL,NULL,FILE_BEGIN); 
		SetEndOfFile(this->Hfile);
		return 1;
	}
	this->error = L"File is not open";
	return 0;
}
ULONGLONG WinFile::GetFileSize( void ){
	if( this->isOpen ){
		LARGE_INTEGER li;
		GetFileSizeEx(this->Hfile,&li);
		return li.QuadPart;
	}
	this->error = L"File is not open";
	return 0;
}
bool WinFile::SetEndFile(const ULONGLONG & byte, const int& start){

	if( this->isOpen ){
		LARGE_INTEGER li;
		li.QuadPart = byte;
		if(SetFilePointerEx(this->Hfile,li,NULL,start)){
			SetEndOfFile(this->Hfile);
			return 1;
		}
	}
	this->error = L"File is not open";
	return 0;
}
DWORD WinFile::GetRW(){
	return this->rw;
}
int WinFile::Delete_File( const std::wstring& path){
	if(!DeleteFile(path.c_str()))
	{
		return GetLastError();
	}
	return 0;
}
ULONGLONG WinFile::GetFileSize( const std::wstring& path ){

	ULONGLONG nFileLen = 0;
	WIN32_FILE_ATTRIBUTE_DATA fData;
	int res = GetFileAttributesEx(path.c_str(), GetFileExInfoStandard, &fData);
	if (res){
		nFileLen = (fData.nFileSizeHigh * (MAXDWORD+1)) + fData.nFileSizeLow;
	}
	return nFileLen;
}
std::wstring WinFile::GetTextError(){
	return this->error;
}
int WinFile::GetError(){
	return this->LastErrorCode;
}

bool WinFile::Fread( std::string& data){
	DWORD len = this->GetFileSize();
	data.clear();
	data.resize(len+1);
	return this->Fread(&data[0],len);
}
bool WinFile::Fread( std::wstring& data){
	DWORD len = this->GetFileSize();
	data.clear();
	data.resize((len/2)+1);
	return this->Fread(&data[0],len);
}
bool WinFile::Fread( std::string& data, const DWORD& Characters){
	data.clear();
	data.resize(Characters+1);
	return this->Fread(&data[0],Characters);
}
bool WinFile::Fread( std::wstring& data,  const DWORD& Characters){
	data.clear();
	data.resize(Characters+1);
	return this->Fread(&data[0],Characters*2);
}
bool WinFile::Fread( LPVOID lpBuf,const DWORD& Byte){
	return ReadFile(this->Hfile,lpBuf,Byte,&this->rw,NULL);
}
bool WinFile::Fread( int& data){
	return this->Fread(&data,sizeof(int));
}
bool WinFile::Fread( bool& data){
	return this->Fread(&data,sizeof(bool));
}
bool WinFile::Fread( float& data){
	return this->Fread(&data,sizeof(float));
}
bool WinFile::Fread( LONG& data){
	return this->Fread(&data,sizeof(LONG));
}
bool WinFile::Fread( LONGLONG& data){
	return this->Fread(&data,sizeof(LONGLONG));
}
bool WinFile::Fread( ULONGLONG& data){
	return this->Fread(&data,sizeof(ULONGLONG));
}
bool WinFile::Fread( DWORD& data){
	return this->Fread(&data,sizeof(DWORD));
}

bool WinFile::Fwrite( std::string& data){
	return this->Fwrite(&data[0],data.length());
}
bool WinFile::Fwrite( std::wstring& data){
	return this->Fwrite(&data[0],data.length()*2);
}
bool WinFile::Fwrite( LPVOID lpBuf,const DWORD & Byte){
	return WriteFile( this->Hfile, lpBuf,  Byte, &this->rw , NULL);
}
bool WinFile::Fwrite( int& data){
	return this->Fwrite(&data,sizeof(int));
}
bool WinFile::Fwrite( bool& data){
	return this->Fwrite(&data,sizeof(bool));
}
bool WinFile::Fwrite( float& data){
	return this->Fwrite(&data,sizeof(float));
}
bool WinFile::Fwrite( LONG& data){
	return this->Fwrite(&data,sizeof(LONG));
}
bool WinFile::Fwrite( LONGLONG& data){
	return  this->Fwrite(&data,sizeof(LONGLONG));
}
bool WinFile::Fwrite( ULONGLONG& data){
	return this->Fwrite(&data,sizeof(ULONGLONG));
}
bool WinFile::Fwrite( DWORD& data){
	return this->Fwrite(&data,sizeof(DWORD));
}
bool WinFile::Fwrite( int&& data){
	return this->Fwrite(&data,sizeof(int));
}
bool WinFile::Fwrite( bool&& data){
	return this->Fwrite(&data,sizeof(bool));
}
bool WinFile::Fwrite( float&& data){
	return this->Fwrite(&data,sizeof(float));
}
bool WinFile::Fwrite( LONG&& data){
	return this->Fwrite(&data,sizeof(LONG));
}
bool WinFile::Fwrite( LONGLONG&& data){
	return  this->Fwrite(&data,sizeof(LONGLONG));
}
bool WinFile::Fwrite( ULONGLONG&& data){
	return this->Fwrite(&data,sizeof(ULONGLONG));
}
bool WinFile::Fwrite( DWORD&& data){
	return this->Fwrite(&data,sizeof(DWORD));
}

int WinFile::Copy_File( const std::wstring& from , const std::wstring& to, int& StatusBar , bool& Stop ){

	DWORD Attr = 0;
	DWORD ReadWrite = 0;

	HANDLE hFile_1 = CreateFile(from.c_str(),GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,Attr,NULL);
	if( !hFile_1 || hFile_1 == INVALID_HANDLE_VALUE){
		return GetLastError();
	}

	HANDLE hFile_2 = CreateFile(to.c_str(),GENERIC_WRITE,NULL,NULL,CREATE_NEW,FILE_ATTRIBUTE_NORMAL,NULL);
	if( !hFile_2 || hFile_2 == INVALID_HANDLE_VALUE){
		return GetLastError();
	}


	ULONGLONG ReadSize = 0,FileSize = 0;
	LARGE_INTEGER li;
	GetFileSizeEx(hFile_1,&li);
	FileSize = li.QuadPart;


	DWORD BufSize = 0;
	if( FileSize < 10485760 /*10mb*/){
		BufSize = FileSize;
	}else{
		BufSize = 10485760;
	}
	byte * Data = NULL;
	Data = (byte*)malloc(BufSize); 
	if(!Data){
		DeleteFile(to.c_str());
		CloseHandle(hFile_1);
		CloseHandle(hFile_2);
		return -1;
	}

	int Procent = 0;

	while (ReadSize < FileSize)
	{
		if(!ReadFile(hFile_1,Data,BufSize,&ReadWrite,NULL)){
			free(Data);
			CloseHandle(hFile_1);
			CloseHandle(hFile_2);
			DeleteFile(to.c_str());		
			return GetLastError();
		}
		ReadSize += ReadWrite;
		if(!WriteFile(hFile_2,Data,ReadWrite,&ReadWrite,NULL)){
			free(Data);
			CloseHandle(hFile_1);
			CloseHandle(hFile_2);
			DeleteFile(to.c_str());
			return GetLastError();
		}

		if(!Stop){
			free(Data);
			CloseHandle(hFile_1);
			CloseHandle(hFile_2);
			DeleteFile(to.c_str());
			return 0;
		}

		if( ((ReadSize * 100)/ FileSize) > StatusBar){
			StatusBar++;
		}
	}
	StatusBar = 100;


	free(Data);
	CloseHandle(hFile_1);
	CloseHandle(hFile_2);

	return 0;


}
int WinFile::Copy_File( const std::wstring& from, const std::wstring& to ){

	DWORD Attr = 0;
	DWORD ReadWrite = 0;

	HANDLE hFile_1 = CreateFile(from.c_str(),GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,Attr,NULL);
	if( !hFile_1 || hFile_1 == INVALID_HANDLE_VALUE){
		return GetLastError();
	}

	HANDLE hFile_2 = CreateFile(to.c_str(),GENERIC_WRITE,NULL,NULL,CREATE_NEW,FILE_ATTRIBUTE_NORMAL,NULL);
	if( !hFile_2 || hFile_2 == INVALID_HANDLE_VALUE){
		return GetLastError();
	}


	ULONGLONG ReadSize = 0,FileSize = 0;
	LARGE_INTEGER li;
	GetFileSizeEx(hFile_1,&li);
	FileSize = li.QuadPart;


	DWORD BufSize = 0;
	if( FileSize < 10485760 /*10mb*/){
		BufSize = FileSize;
	}else{
		BufSize = 10485760;
	}
	byte * Data = NULL;
	Data = (byte*)malloc(BufSize); 
	if(!Data){
		DeleteFile(to.c_str());
		CloseHandle(hFile_1);
		CloseHandle(hFile_2);
		return -1;
	}



	while (ReadSize < FileSize)
	{
		if(!ReadFile(hFile_1,Data,BufSize,&ReadWrite,NULL)){
			free(Data);
			CloseHandle(hFile_1);
			CloseHandle(hFile_2);
			DeleteFile(to.c_str());
			return GetLastError();
		}
		ReadSize += ReadWrite;
		if(!WriteFile(hFile_2,Data,ReadWrite,&ReadWrite,NULL)){
			free(Data);
			CloseHandle(hFile_1);
			CloseHandle(hFile_2);
			DeleteFile(to.c_str());
			return GetLastError();
		}

	}



	free(Data);
	CloseHandle(hFile_1);
	CloseHandle(hFile_2);
	return 0;
}
int WinFile::Rename_File(const std::wstring& from, const std::wstring&  to ){
	if(MoveFile(from.c_str(),to.c_str())){
		return 0;
	}
	return GetLastError();
}
int WinFile::file_get_contents(const std::wstring& path, std::wstring& result){
	WinFile wf;
	if(wf.Fopen(path,L"r")){
		if(wf.Fread(result)){
			wf.Fclose();
			return 0;
		}
	}
	return wf.GetError();
}
int WinFile::file_put_contents(const std::wstring &path, std::wstring &data){
	WinFile wf;
	if(wf.Fopen(path,L"w")){
		if(wf.Fwrite(data)){
			wf.Fclose();
			return 0;
		}
	}
	return wf.GetError();
}
int WinFile::file_get_contents(const std::wstring& path, std::string& result){
	WinFile wf;
	if(wf.Fopen(path,L"r")){
		if(wf.Fread(result)){
			wf.Fclose();
			return 0;
		}
	}
	return wf.GetError();
}
int WinFile::file_put_contents(const std::wstring &path, std::string &data){
	WinFile wf;
	if(wf.Fopen(path,L"w")){
		if(wf.Fwrite(data)){
			wf.Fclose();
			return 0;
		}
	}
	return wf.GetError();
}